# Copyright 2010 Canonical Ltd. All rights reserved.

"""Link external sourcecode."""

import os
import re
import sys
import errno
import hashlib
import subprocess

import urllib

from bzrlib.branch import Branch
from bzrlib.config import LocationConfig
from bzrlib.plugin import load_plugins
load_plugins()
from bzrlib.plugins.launchpad.account import get_lp_login


def get_ubunet_branch_config(rootdir):
    """
    Retrieves the sourcedeps configuration for an ubunet source dir.
    Returns a dict of (branch, revspec) tuples, keyed by branch name.
    """
    config_file = os.path.join(rootdir, 'config-manager.txt')
    branches = {}
    with open(config_file, 'r') as stream:
        for line in stream:
            line = line.split('#')[0].strip()
            match = re.match(r'./sourcecode/(\S+)\s+'
                              'bzr\+ssh://bazaar.launchpad.net/([^;]+)'
                              '(?:;revno=(\d+))?', line)
            if match:
                name, branch, revno = match.group(1, 2, 3)
                if revno is None:
                    revspec = "-1"
                else:
                    revspec = revno
                branches[name] = (branch, revspec)
    return branches


SOURCECODE_DIR = 'sourcecode'


def main(parent, target, verbose):
    """Do the deed."""
    parent_dir = os.path.realpath(os.path.join(parent, SOURCECODE_DIR))
    target_dir = os.path.abspath(os.path.join(target, SOURCECODE_DIR))

    try:
        os.makedirs(parent_dir)
    except OSError, e:
        if e.errno != errno.EEXIST:
            raise
    bzr_config = LocationConfig(parent_dir)

    branches = sorted(get_ubunet_branch_config(target).items())
    for branch_name, (quoted_branch_spec, revspec) in branches:
        revno = int(revspec)

        # qualify mirror branch name with hash of remote repo path to deal
        # with changes to the remote branch URL over time
        branch_spec_digest = hashlib.sha1(quoted_branch_spec).hexdigest()
        branch_directory = "%s_%s" % (branch_name, branch_spec_digest)

        source_path = os.path.join(parent_dir, branch_directory)
        destination_path = os.path.join(target_dir, branch_name)

        # Remove leftover symlinks/stray files.
        try:
            os.remove(destination_path)
        except OSError, e:
            if e.errno != errno.EISDIR and e.errno != errno.ENOENT:
                raise

        sys.stderr.write('%30s: ' % (branch_name,))
        sys.stderr.flush()

        # Create the local mirror branch if it doesn't already exist
        if not os.path.exists(source_path):
            username = get_lp_login(bzr_config) or get_lp_login()
            if username is None:
                raise RuntimeError("Unable to determine launchpad login")
            quoted_username = urllib.quote(username)
            branch_url = ("bzr+ssh://%s@bazaar.launchpad.net/%s" %
                          (quoted_username, quoted_branch_spec))
            subprocess.check_call(['bzr', 'branch',
                                   '--', branch_url, source_path])

        source_branch = Branch.open(source_path)

        # Freshen the source branch if required (-1 means we don't care).
        if revno != -1 and revno > source_branch.revno():
            subprocess.check_call(['bzr', 'pull', '-r', str(revno),
                                   '-d', source_path])

        if os.path.exists(destination_path):
            # Overwrite the destination with the appropriate revision.
            subprocess.check_call(['bzr', 'pull', '--overwrite',
                                   '-r', str(revno),
                                   '-d', destination_path, '--', source_path])
        else:
            # Do a new checkout.
            subprocess.check_call(['bzr', 'checkout', '--hardlink',
                                   '-r', str(revno),
                                   '--', source_path, destination_path])

        # Check the state of the destination branch.
        destination_branch = Branch.open(destination_path)
        destination_revno = destination_branch.revno()

        if verbose:
            sys.stderr.write('checked out r%d of %s\n' %
                             (destination_revno, branch_name))
            sys.stderr.flush()

        # Shouldn't happen, but bzr has lots of bugs.
        if revno != -1 and destination_revno != revno:
            raise RuntimeError("Expected revno %d but got revno %d" %
                               (revno, destination_revno))
