# -*- coding: utf-8 -*-

# Copyright 2011, 2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Actions for the Contacts page of the Ubuntu One website."""

from sst.actions import *
from loading import *
from data.contact import Contact

def open():
    click_contacts_tab()
    switch_to_window()

def click_contacts_tab():
    click_link(_get_contacts_tab_link())

def _get_contacts_tab_link():
    return get_element_by_css('#main-nav .contacts')

def click_contacts_bottom_link():
    click_link(_get_contacts_bottom_link())

def _get_contacts_bottom_link():
    return get_element_by_css('footer nav .contacts')

def assert_page_title():
    """Assert that the title of the page is the expected."""
    assert_title(u'Ubuntu One : Contacts')

def assert_page_heading1():
    heading1_element = get_element(tag='h1')
    assert_text(heading1_element, 'Contacts')

def add_contact(contact):
    """Add a new contact.

    Keyword arguments:
    contact -- The contact that will be added.

    """
    link_add = get_element(css_class='add-button')
    click_link(link_add)
    wait_for_action_to_complete()
    _fill_contact(contact)
    link_save = get_elements(css_class='save-button')[0]
    click_element(link_save)
    wait_for_action_to_complete()

def edit_contact(edited_contact):
    """Edit the information of the opened contact.

    Keyword arguments:
    edited_contact -- The contact with the edited information.

    """
    link_edit = get_element(css_class='edit-button')
    click_element(link_edit)
    wait_for_action_to_complete()
    _fill_contact(edited_contact)
    link_save = get_elements(css_class='save-button')[0]
    click_element(link_save)
    wait_for_action_to_complete()

def _fill_contact(contact):
    """Fill the contact information.

    Keyword arguments:
    contact -- The contact with the information that will be filled.

    """
    if contact.name:
        _fill_contact_name(contact.name)
    _fill_contact_emails(contact.emails)
    _fill_contact_phones(contact.phones)
    _fill_contact_addresses(contact.addresses)
    _fill_contact_instant_messaging_addresses(
        contact.instant_messaging_addresses)
    _fill_contact_urls(contact.urls)
    if contact.role:
        _fill_contact_role(contact.role)
    if contact.about_information:
        _fill_contact_about_information(contact.about_information)

def _fill_contact_name(name):
    """Fill the contact name information.

    Keyword arguments:
    name -- The name of the contact.

    """
    if name.title:
        write_textfield('names-title', name.title)
    if name.first_name:
        write_textfield('names-first_name', name.first_name)
    if name.middle_name:
        write_textfield('names-middle_name', name.middle_name)
    if name.last_name:
        write_textfield('names-last_name', name.last_name)
    if name.suffix:
        write_textfield('names-suffix', name.suffix)
    
def _fill_contact_emails(emails):
    """Fill the contact emails information.

    Keyword arguments:
    emails -- A sequence with the emails of the contact.

    """
    for index, email in enumerate(emails):
        set_dropdown_value('%d-email_addresses-description' % index, 
                        email.description)
        write_textfield('%d-email_addresses-address' % index, email.address)
        click_element('add-email-address')

def _fill_contact_phones(phones):
    """Fill the contact phones information.

    Keyword arguments:
    phones -- A sequence with the phones of the contact.

    """
    for index, phone in enumerate(phones):
        set_dropdown_value('%d-phone_numbers-description' % index, phone.description)
        write_textfield('%d-phone_numbers-number' % index, phone.number)
        click_element('add-phone-number')
        
def _fill_contact_addresses(addresses):
    """Fill the contact addresses information.

    Keyword arguments:
    addresses -- A sequence with the addresses of the contact.
    
    """
    for index, address in enumerate(addresses):
        set_dropdown_value('%d-addresses-description' % index, address.description)
        write_textfield('%d-addresses-address1' % index, address.address1)
        write_textfield('%d-addresses-address2' % index, address.address2)
        write_textfield('%d-addresses-pobox' % index, address.po_box)
        write_textfield('%d-addresses-city' % index, address.city)
        write_textfield('%d-addresses-state' % index, address.state)
        write_textfield('%d-addresses-postalcode' % index, address.postal_code)
        write_textfield('%d-addresses-country' % index, address.country)
        click_element('add-postal-address')

def _fill_contact_instant_messaging_addresses(instant_messaging_addresses):
    """Fill the contact instant messaging information.

    Keyword arguments:
    instant_messaging_addresses -- A sequence of instant messaging addresses.

    """
    for index, address in enumerate(instant_messaging_addresses):
        set_dropdown_value('%d-im_addresses-description' % index, address.description)
        write_textfield('%d-im_addresses-address' % index, address.address)
        click_element('add-im-address')

def _fill_contact_urls(urls):
    """Fill the contact URLs information.

    Keyword arguments:
    urls -- A sequence with the URLs of the contact.

    """
    for index, url in enumerate(urls):
        write_textfield('%d-urls-description' % index, url.description)
        write_textfield('%d--urls-address' % index, url.url)
        click_element('add-url')

def _fill_contact_role(role):
    """Fill the contact role information.

    Keyword arguments:
    role -- The job role information of the contact.

    """
    if role.company:
        write_textfield('work-company', role.company)
    if role.department:
        write_textfield('work-department', role.department)
    if role.job_title:
        write_textfield('work-job_title', role.job_title)
    if role.manager_name:
        write_textfield('work-manager_name', role.manager_name)
    if role.assitant_name:
        write_textfield('work-assistant_name', role.assistant_name)

def _fill_contact_about_information(about_information):
    """Fill the additional information about the contact.

    Keyword arguments:
    about_information -- The additional information about the contact.

    """
    if about_information.nickname:
        write_textfield('personal-nick_name', about_information.nickname)
    if about_information.birth_date:
        write_textfield('personal-birth_date', about_information.birth_date)
    if about_information.spouse_name:
        write_textfield('personal-spouse_name', about_information.spouse_name)
    if about_information.wedding_date:
        write_textfield('personal-wedding_date',
                        about_information.wedding_date)
    if about_information.notes:
        write_textfield('personal-notes', about_information.notes)

def assert_contact_information(contact):
    """Assert that the information of the displayed contact is the expected.

    Keyword arguments:
    contact -- The expected contact information.

    """
    _assert_contact_name(contact.name)
    _assert_contact_emails(contact.emails)
    contact_identifier = get_contact_identifier()
    _assert_contact_in_list(contact_identifier, contact)
    # TODO _assert_contact_phones(contact.phones)
    # TODO _assert_contact_addresses(contact.addresses)
    # TODO _assert_contact_instant_messaging_addresses(
    # contact.instant_messaging_addresses)
    # TODO _assert_contact_urls(contact.urls)
    # TODO _assert_contact_role(contact.role)
    # TODO _assert_contact_about_information(contact.about_information)

def get_contact_identifier():
    """Get the identifier of the opened contact."""
    button_edit = get_element_by_css('.edit-button')
    return button_edit.get_attribute('id')

def _get_element_contact_record(identifier):
    """Get the contact record element from the contacts list.

    Keyword arguments:
    identifier -- The identifier of the contact.
    
    """
    return get_element(id=identifier, css_class='contact-record')

def _assert_contact_in_list(contact_identifier, contact_information):
    """Assert that the contact appears on the right place on the contacts list.

    Keyword arguments:
    contact_identifier -- The identifier of the contact.
    contact_information -- The expected contact information.

    """
    # TODO to assert that the contact is under the right letter we need it to
    # have separate divs for each letter.
    # first_letter = _get_contact_letter_in_list(contact.name)
    display_name = _get_contact_display_name_in_list(contact_information.name)
    element_contact = _get_element_contact_record(contact_identifier)
    element_name = element_contact.find_element_by_css_selector('.name a')
    assert_text(element_name, display_name)
    if contact_information.emails:
        element_email = element_contact.find_element_by_css_selector('.email')
        assert_text(element_email, contact_information.emails[0].address)

def _get_contact_display_name_in_list(name):
    """Get the expected name to display in the contacts list.

    Keyword arguments:
    name -- The name information of the contact.

    """
    if name and (name.last_name or name.first_name):
        names = []
        if name.last_name:
            names.append(name.last_name)
        if name.first_name:
            names.append(name.first_name)
        display_name = ', '.join(names)
    else:
        display_name = 'New contact'
    return display_name

def _get_contact_letter_in_list(name):
    """Get the letter under which it's expected to find the contact in the 
    list.

    Keyword arguments:
    name -- The name information of the contact.

    """
    if name and (name.last_name or name.first_name):
        if name.last_name:
            first_letter = name.last_name[0]
        else:
            first_letter = name.first_name[0]
    else:
        first_letter = '0' # TODO ask if this is the correct behavior.

def _assert_contact_name(name):
    """Assert that the name of the displayed contact is the expected.

    Keyword arguments:
    name -- The name information of the contact.

    """
    if not name:
        display_name = 'New contact'
    else:
        names = []
        if name.title:
            names.append(name.title)
        if name.first_name:
            names.append(name.first_name)
        if name.middle_name:
            names.append(name.middle_name)
        if name.last_name:
            names.append(name.last_name)
        if name.suffix:
            names.append(name.suffix)
        display_name = ' '.join(names)
    label = get_element_by_css('#names>h2')
    assert_text(label, display_name)

def _assert_contact_emails(emails):
    """Assert that the email addresses of the displayed contact are the 
    expected.

    Keyword arguments:
    emails -- A sequence with the expected email addresses.

    """
    elements_email_descriptions = get_elements_by_css('#email_addresses dt')
    elements_email_addresses = get_elements_by_css('#email_addresses a')
    for index, email in enumerate(emails):
        assert_text(elements_email_descriptions[index], '%s:' % email.description)
        assert_text(elements_email_addresses[index], email.address)

def delete_contact():
    """Delete the opened contact."""
    contact_identifier = get_contact_identifier()
    element_contact = _get_element_contact_record(contact_identifier)
    # Click the contact to make the browser scroll to it. This was causing
    # a WebDriverException.
    element_contact.click()
    element_contact_checkbox = element_contact. \
        find_element_by_css_selector('.contact-delete-checkbox')
    set_checkbox_value(element_contact_checkbox, True)
    click_element('delete-selected', wait=False)
    accept_alert('Delete selected contacts?')
    wait_for_action_to_complete()

def assert_contact_exists(identifier):
    """Assert that a contact exists.

    Keyword arguments:
    identifier -- The identifier of the contact.
    """
    _get_element_contact_record(identifier)
