# -*- coding: utf-8 -*-

# Copyright 2011, 2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Actions for the Files page of the Ubuntu One website."""

from sst.actions import *
from sst.actions import _get_elem
from loading import *
from config import environment
import urlparse


def open():
    click_files_tab()
    switch_to_window()


def click_files_tab():
    click_link(_get_files_tab_link())


def _get_files_tab_link():
    return get_element_by_css('#main-nav .files')


def click_dashboard_tab():
    click_link(_get_dashboard_tab_link())


def _get_dashboard_tab_link():
    return get_element_by_css('#main-nav .dashboard')


def click_files_bottom_link():
    click_link(_get_files_bottom_link())


def _get_files_bottom_link():
    return get_element_by_css('footer nav .files')


def assert_page_title():
    """Assert that the title of the page is the expected."""
    assert_title(u'Ubuntu One : My Storage')


def assert_page_heading1():
    heading1_element = get_element(tag='h1')
    assert_text(heading1_element, 'Files')


def upload_file(path):
    """Upload a file.

    Keyword arguments:
    path -- The path of the file to upload.

    """
    click_upload_file()
    input_file = _get_elem('upload-overlay-file')
    input_file.send_keys(path)
    click_button('upload-overlay-upload-button')
    # TODO set a waiting time proportional to the size of the file to upload.
    wait_for_action_to_complete()


def click_upload_file():
    """Click the Upload file button and wait for the upload overlay
    to appear.

    """
    click_element('upload-file-button', wait=False)
    upload_overlay = get_element(id='upload-overlay')
    wait_for(upload_overlay.is_displayed)


def assert_file_exists(name):
    """Assert that a file exists on the Ubuntu One storage.

    Keyword arguments:
    name -- The name of the file.

    """
    # XXX This is not working with get_element. Selenium throws an error.
    get_element_by_xpath('//td[@class="files-td-name" and @id="%(name)s"]'
                         % {'name': name})


def create_new_folder(name):
    """Create a new folder on the Ubuntu One storage service.

    The folder will be created as a child of the currently selected one.

    Keyword arguments:
    name -- The name of the folder.

    """
    click_new_folder()
    write_textfield('new-folder-overlay-name', name)
    click_button('new-folder-overlay-create-button')
    wait_for_action_to_complete()


def click_new_folder():
    """Click the New folder link and wait for the new folder overlay
    to appear.

    """
    click_element('new-folder-button', wait=False)
    new_folder_overlay = get_element(id='new-folder-overlay')
    wait_for(new_folder_overlay.is_displayed)


def get_file_row_element(name):
    """Get the HTML row of a file.

    Keyword arguments:
    name -- The name of the file.

    """
    return get_element_by_xpath('//td[@class="files-td-name" and ' \
                                    '@id="%(name)s"]/..' % {'name': name})


def _get_file_identifier(name):
    """Get the identifier of a file.

    Keyword arguments:
    name -- The name of the file.

    """
    element_file_row = get_file_row_element(name)
    return element_file_row.get_attribute('id')


def get_file_upload_date(name):
    """Get the upload date of a file.

    Keyword arguments:
    name -- The name of the file.

    """
    element_file_row = get_file_row_element(name)
    return element_file_row.find_element_by_xpath('td[@class="files-td-date"]'
                                                  '/span').text


def get_file_size(name):
    """Get the size of a file.

    Keyword arguments:
    name -- The name of the file.

    """
    element_file_row = get_file_row_element(name)
    return element_file_row. \
        find_element_by_xpath('td[@class="files-td-size"]').text


def show_file_details(name):
    if not _are_details_shown:
        _click_file_more(name)


def _are_details_shown(name):
    identifier = _get_file_identifier(name)
    element_details = get_element(id='%s-details' % identifier)
    return element_details.is_displayed()


def _click_file_more(name):
    """Click the link to show or hide the file details.

    Keyword arguments:
    name -- The name of the file.

    """
    element_file_row = get_file_row_element(name)
    element_more = element_file_row.find_element_by_xpath(
        'td[@class="files-td-expander expand-row-link"]/a')
    click_element(element_more)


def assert_file_information_is_shown(name):
    """Assert that the full information of a file is shown.

    Keyword arguments:
    name -- The name of the file.

    """
    assert _are_details_shown(name)
    identifier = _get_file_identifier(name)
    assert _get_link_publish_file(identifier).is_displayed()
    assert _get_link_download_file(identifier).is_displayed()
    assert _get_link_delete_file(identifier).is_displayed()


def _get_link_publish_file(identifier):
    """Get the Publish File link element.

    Keyword arguments:
    identifier -- The identifier of the file.

    """
    return get_element(id='%s-publish' % identifier,
                       css_class='u1-files-publish-link')


def _get_link_stop_publishing_file(identifier):
    """Get the Stop Publish File link element.

    Keyword arguments:
    identifier -- The identifier of the file.

    """
    return get_element(id='%s-publish' % identifier,
                       css_class='u1-files-stop-publishing-link')


def _get_link_download_file(identifier):
    """Get the Download file link element.

    Keyword arguments:
    identifier -- The identifier of the file.

    """
    files_url = environment.get_environment().files_url
    file_url = files_url + str(identifier[3:])
    return get_element(css_class='download-button', href=file_url)


def _get_link_delete_file(identifier):
    """Get the Delete file link element.

    Keyword arguments:
    identifier -- The identifier of the file.

    """
    return get_element(id='%s-delete' % identifier)


def assert_file_type(name, type):
    """Assert that the type of a file is the expected.

    Keyword arguments:
    name -- The name of the file.
    type -- The expected type of the file.

    """
    show_file_details(name)
    # XXX this element requires an identifier.
    element_file_type = get_element_by_xpath('id("%s-details")//li[3]/strong'
                                             % _get_file_identifier(name))
    assert_text(element_file_type, type)


def publish_file(name):
    """Publish a file.

    Keyword arguments:
    name -- The name of the file.

    """
    show_file_details(name)
    identifier = _get_file_identifier(name)
    click_element(_get_link_publish_file(identifier))
    wait_for_action_to_complete()


def assert_file_published(name):
    """Assert that a folder is shared.

    Keyword arguments:
    name -- The name of the published file.

    """
    show_file_details(name)
    identifier = _get_file_identifier(name)
    _get_link_stop_publishing_file(identifier)
    fails(_get_link_publish_file, identifier)
    get_public_url(name)


def get_public_url(name):
    """Get the public URL of a published file.

    Keyword arguments:
    name -- The name of the file.

    """
    show_file_details(name)
    # XXX this element requires an identifier.
    input_public_url = get_element_by_xpath('id("%s-details")//'
                                           'input[@class="public_url"]'
                                           % _get_file_identifier(name))
    public_url = input_public_url.get_attribute('value')
    parsed_url = urlparse.urlparse(public_url)
    base_url = public_url[:-len(parsed_url.path)]
    path = parsed_url.path
    return base_url, path


def go_to_public_url(name):
    base_url, path = get_public_url(name)
    set_base_url(base_url)
    go_to(path)


def stop_publishing_file(name):
    """Stop publishing a file.

    Keyword arguments:
    name -- The name of the file.

    """
    show_file_details(name)
    identifier = _get_file_identifier(name)
    click_element(_get_link_stop_publishing_file(identifier))
    wait_for_action_to_complete()


def assert_file_not_published(name):
    """Assert that a folder is shared.

    Keyword arguments:
    name -- The name of the published file.

    """
    show_file_details(name)
    identifier = _get_file_identifier(name)
    fails(_get_link_stop_publishing_file, identifier)
    _get_link_publish_file(identifier)
    fails(get_public_url, name)


def click_download_file(name):
    """Download a file stored on the Ubuntu One service.

    Keyword arguments:
    name -- The name of the file.

    """
    show_file_details(name)
    identifier = _get_file_identifier(name)
    click_link(_get_link_download_file(identifier))


def get_download_url(name):
    """Get the downlaod link of a file stored on the Ubuntu One service.

    Keyword arguments:
    name -- The name of the file.
    """
    show_file_details(name)
    identifier = _get_file_identifier(name)
    return _get_link_download_file(identifier).get_attribute('href')


def _get_link_share_folder(identifier):
    """Get the Share folder link element.

    Keyword arguments:
    identifier -- The identifier of the folder.

    """
    return get_element(id='%s-share' % identifier)


def assert_folder_information_is_shown(name):
    """Assert that the full information of a folder is shown.

    Keyword arguments:
    name -- The name of the file.

    """
    assert _are_details_shown(name)
    identifier = _get_file_identifier(name)
    assert _get_link_share_folder(identifier).is_displayed()
    assert _get_link_delete_file(identifier).is_displayed()


def open_folder(name):
    """Open a folder from the current path.

    Keyword arguments:
    name -- The name of the folder.

    """
    element_row = get_file_row_element(name)
    link_folder = element_row.find_element_by_css_selector('a')
    click_element(link_folder)
    wait_for_action_to_complete()


def assert_breadcrumb(*path):
    """Assert that the breadcrumb path is the expected.

    Keyword arguments:
    path -- A list with the expected breadcrumb path.

    """
    breadcrumbs = get_elements(css_class='breadcrumb')
    assert_text(breadcrumbs[0], 'My Storage')
    map(assert_text, breadcrumbs[1:], path)


def go_to_breadcrumb(*path):
    """Open a folder from the breadcrumb path.

    Keyword arguments:
    path -- The breadcrumb path of the folder to open. If no path is specified,
        the top folder 'My Storage' will be opened.

    """
    url = '/files/'
    escaped_path = [folder.replace(' ', '%20') for folder in path]
    if path:
        url += 'f/' + '/'.join(escaped_path)
    element_breadcrumb_link = get_element_by_css('.breadcrumb[href="%s"]'
                                                 % url)
    click_link(element_breadcrumb_link)
    wait_for_action_to_complete()


def _fill_sharing_dialog(email, share_name, allow_modification=True):
    """Fill the sharing dialog.

    Keyword arguments:
    email -- The email address of the person who will receive the shared
        folder.
    share_name -- The name to set for the shared folder.
    allow_modification -- Indicates if the receipient of the shared folder
        will be allowed to modify it. The default value is True.

    """
    write_textfield('sharing-overlay-email', email)
    write_textfield('sharing-overlay-name', share_name)
    set_checkbox_value('sharing-overlay-readwrite', allow_modification)


def share_folder(name, email, share_name, allow_modification=True):
    """Share a folder.

    Keyword arguments:
    name -- The name of the folder to share.
    email -- The email address of the person who will receive the shared
        folder.
    share_name -- The name to set for the shared folder.
    allow_modification -- Indicates if the receipient of the shared folder
        will be allowed to modify it. The default value is True.

    """
    show_file_details(name)
    identifier = _get_file_identifier(name)
    link_share_folder = _get_link_share_folder(identifier)
    click_element(link_share_folder)
    sharing_overlay = get_element(tag='*', id='sharing-overlay')
    wait_for(sharing_overlay.is_displayed)
    _fill_sharing_dialog(email, share_name, allow_modification)
    click_button('sharing-overlay-share-button')
    wait_for_action_to_complete()


def assert_folder_shared(name, email, date, allow_modification=True,
                         accepted=False):
    """Assert that a folder is shared.

    Keyword arguments:
    name -- The name of the shared folder.
    email -- The email address of the person who received the shared folder.
    date -- The date the folder was shared.
    allow_modification -- Indicates if the recipient of the shared folder is
        allowed to modify it. The default value is True.
    accepted -- Indicates if the folder has been accepted by the recipient.
        The default value is False.

    """
    show_file_details(name)
    expected_shared_information = \
        _build_shared_information_string(name, email, date, allow_modification,
                                         accepted)
    folder_shared_information = _get_shared_with_information(name)
    assert expected_shared_information in folder_shared_information


def _build_shared_information_string(name, email, date,
                                    allow_modification=True, accepted=False):
    """Build the string with the sharing information of a folder.

    Keyword arguments:
    name -- The name of the shared folder
    email -- The email address of the person who received the shared folder.
    date -- The date the folder was shared.
    allow_modification -- Indicates if the recipient of the shared folder is
        allowed to modify it. The default value is True.
    accepted -- Indicates if the folder has been accepted by the recipient.
        The default value is False.

    """
    modification_information = 'view'
    if allow_modification:
        modification_information += '/change'
    shared_information = 'Shared with %(email)s to %(permissions)s ' \
                                  'files on %(date)s' \
                                  % {'email': email,
                                     'permissions': modification_information,
                                     'date': date.strftime('%b-%d-%Y')}
    if not accepted:
        shared_information += ' (waiting to be accepted)'
    shared_information += ' Stop sharing'
    return shared_information


def _get_shared_with_information(name):
    """Get the information of the shared folder.

    Keyword arguments:
    name -- The name of the shared folder.

    Returns a list with the information of all the shares for the folder.

    """
    elements_shared_information = _get_shared_with_elements(name)
    return [element.text for element in elements_shared_information]


def _get_shared_with_elements(name):
    """Get the HTML elements with the information of the shared folder.

    Keyword arguments:
    name -- The name of the shared folder.

    Returns a list with the HTML elements of all the shares for the folder.

    """
    identifier = _get_file_identifier(name)
    elements_shared_information = get_elements_by_css('#%s-details '
                                                      'li.shared-with'
                                                      % identifier)
    return elements_shared_information


def stop_sharing_folder(name, email, date, allow_modification=True,
                        accepted=False):
    """Stop sharing a folder.

    Keyword arguments:
    name -- The name of the shared folder.
    email -- The email address of the person who received the shared folder.
    date -- The date the folder was shared.
    allow_modification -- Indicates if the recipient of the shared folder is
        allowed to modify it. The default value is True.
    accepted -- Indicates if the folder has been accepted by the recipient.
        The default value is False.

    """
    show_file_details(name)
    link_stop_sharing = _get_stop_sharing_folder_link(name, email, date,
                                                      allow_modification,
                                                      accepted)
    click_element(link_stop_sharing, wait=False)
    accept_alert()
    wait_for_action_to_complete()


def _get_stop_sharing_folder_link(name, email, date, allow_modification=True,
                        accepted=False):
    """Get the link to stop sharing a folder.

    Keyword arguments:
    name -- The name of the shared folder.
    email -- The email address of the person who received the shared folder.
    date -- The date the folder was shared.
    allow_modification -- Indicates if the recipient of the shared folder is
        allowed to modify it. The default value is True.
    accepted -- Indicates if the folder has been accepted by the recipient.
        The default value is False.

    """
    identifier = _get_file_identifier(name)
    shared_information = \
        _build_shared_information_string(name, email, date, allow_modification,
                                         accepted)
    elements_shared_information = _get_shared_with_elements(name)
    folder_list = filter(lambda element : element.text == shared_information,
                         elements_shared_information)
    assert folder_list, 'Stop sharing link not found.'
    return folder_list[0].find_element_by_css_selector('a')


def delete_file(name):
    """Delete a file or folder.

    Keyword arguments:
    name -- The name of the file or folder to delete.

    """
    show_file_details(name)
    identifier = _get_file_identifier(name)
    link_delete = _get_link_delete_file(identifier)
    click_element(link_delete, wait=False)
    accept_alert()
    wait_for_action_to_complete()
