# -*- coding: utf-8 -*-

# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Actions for the Music page of the Ubuntu One website."""

from sst.actions import *
from data.song import Song

def open():
    """Open the music page and assert it's title."""
    click_link(_get_music_link())
    go_to('music/player')
    switch_to_window()
    assert_page_title()
    _wait_for_action_to_complete()

def _get_music_link():
    return get_element_by_css('#main-nav .music')

def assert_page_title():
    """Assert that the title of the page is the expected."""
    assert_title(u'Ubuntu One : Music')

def play_song(song):
    _open_songs()
    song_id = _get_song_id(song.title, song.artist, song.album)
    _play_song_by_id(song_id)

def _play_song_by_id(identifier):
    song_play_button = get_element_by_css('tr[data-u1m-songid="%s"] > ' \
                                          'td.c1 > button' \
                                          % identifier)
    click_button(song_play_button)
    assert not song_play_button.is_displayed()

def assert_songs_added_to_queue(songs_list):
    number_of_songs = len(songs_list)
    assert number_of_songs > 0
    _assert_message_displayed()
    _assert_songs_added_to_queue_message(number_of_songs)
    for song in songs_list:
        assert_song_in_queue(song)
    
def _assert_songs_added_to_queue_message(number_of_songs):
    message_content = None
    if number_of_songs == 1:
        message_content = '1 song added to the queue.'
    else:
        message_content = '%d songs added to the queue.'
    _assert_message_content(message_content)

def _assert_message_displayed():
    assert _get_message_element().is_displayed()

def _assert_message_content(message_content):
    assert _get_message_element().text == message_content

def _get_message_element():
    return get_element_by_css('.yui3-message.yui3-info .yui3-widget-bd')

def select_song(song):
    _open_songs()
    song_id = _get_song_id(song.title, song.artist, song.album)
    _select_song_by_id(song_id)

def _select_song_by_id(identifier):
    song_checkbox = get_element_by_css('.c2 > input[value="%s"]' % identifier)
    set_checkbox_value(song_checkbox, True)
    
def _open_songs():
    if not _is_songs_open():
        songs_element = _get_songs_element()
        click_element(songs_element)
        _wait_for_action_to_complete()

def _is_songs_open():
    songs_element = _get_songs_element()
    return _is_active(songs_element)

def _get_songs_element():
    return get_element(css_class='my-songs')

def _get_song_id(title, artist, album):
    song_row_xpath = '//td[@class="c3" and text()="%(title)s"]/' \
                     '../td[@class="c4 artist" and text()="%(artist)s"]/../' \
                     'td[@class="c5 album" and text()="%(album)s"]/..' \
                     % {'title': title, 'artist': artist, 'album': album}
    song_row_element = get_element_by_xpath(song_row_xpath)
    return song_row_element.get_attribute('data-u1m-songid')

def assert_player_title(number_of_song, total_songs):
    player_title_element = get_element_by_css('.player > h2')
    expected_title = 'Now playing %d of %d songs' \
                     % (number_of_song, total_songs)
    assert expected_title == player_title_element.text

def assert_player_playing():
    assert _get_stop_button().is_displayed()
    assert not _get_play_button().is_displayed()

def assert_player_paused():
    assert not _get_stop_button().is_displayed()
    assert _get_play_button().is_displayed()

def _get_stop_button():
    return get_element_by_css('#stop')

def _get_play_button():
    return get_element_by_css('#play')

def assert_current_song(song):
    assert song.title == _get_current_song_title()
    assert song.artist == _get_current_song_artist()
    assert song.album == _get_current_song_album()

def _get_current_song_title():
    current_song_title_element = get_element_by_css('.current-song .title')
    return current_song_title_element.text

def _get_current_song_artist():
    current_song_artist_element = get_element_by_css('.current-song .artist')
    return current_song_artist_element.text

def _get_current_song_album():
    current_song_album_element = get_element_by_css('.current-song .album')
    return current_song_album_element.text

def clear_queue():
    _open_now_playing()
    click_button(_get_clear_queue_button())

def _get_clear_queue_button():
    return get_element(css_class='clear')

def assert_song_in_queue(song):
    _open_now_playing()
    _assert_song(song)

def _assert_song(song):
    song_row_xpath = '//tr[contains(@class, "song-listing")]/' \
                     'td[text()="%(title)s"]/../td[text()="%(artist)s"]/../' \
                     'td[text()="%(album)s"]/..' % {'title': song.title,
                                                    'artist': song.artist,
                                                    'album': song.album}
    get_element_by_xpath(song_row_xpath)

def _open_now_playing():
    if not _is_now_playing_open():
        click_element(_get_now_playing_element())

def _is_now_playing_open():
    now_playing_element = _get_now_playing_element()
    return _is_active(now_playing_element)

def _get_now_playing_element():
    return get_element(css_class='nowplaying')

def _is_active(element):
    element_class = element.get_attribute('class')
    return 'active' in element_class

def _wait_for_action_to_complete():
    """Wait for the last action to complete.

    This function waits for the waiting bar to disappear.
    
    """
    # Added a little sleep because if this function is called to fast before
    # the bar appears tests will fail.
    sleep(3)
    waiting_bar = get_element_by_css('.loading-view > .indicator')
    wait_for(fails, assert_displayed, waiting_bar)

def select_artist(artist):
    _open_artists()
    _click_artist(artist)
    
def _open_artists():
    if not _is_artists_open():
        click_element(_get_artists_element())
        _wait_for_action_to_complete()

def _is_artists_open():
    artists_element = _get_artists_element()
    return _is_active(artists_element)

def _get_artists_element():
    return get_element(css_class='artists')

def _click_artist(artist):
    artist_link_xpath = '//a[@data-u1m-artistid and text()="{0}"]' \
                        .format(artist)
    artist_link = get_element_by_xpath(artist_link_xpath)
    artist_link.click()
    _wait_for_action_to_complete()

def assert_artist_albums(artist, albums_list):
    _assert_albums_view_title('{0} albums'.format(artist))
    for album in albums_list:
        _assert_album(album)

def _assert_albums_view_title(title):
    view_title = get_element_by_css('#album-view > h2')
    assert_text(view_title, title)

def _assert_album(album):
    _get_album_link(album)

def _get_album_link(album):
    album_link_xpath = '//a[@data-u1m-albumid and text()="{0}"]' \
                        .format(album)
    return get_element_by_xpath(album_link_xpath)

def select_album(album):
    _open_albums()
    _click_album(album)
    
def _open_albums():
    if not _is_albums_open():
        click_element(_get_albums_element())
        _wait_for_action_to_complete()

def _is_albums_open():
    albums_element = _get_albums_element()
    return _is_active(albums_element)

def _get_albums_element():
    return get_element(css_class='albums')

def _click_album(album):
    _get_album_link(album).click()
    _wait_for_action_to_complete()

def assert_album_songs(artist, album, songs_title_list):
    _assert_songs_view_title('{0} by {1}'.format(album, artist))
    for song_title in songs_title_list:
        _assert_song(Song(artist, album, song_title))

def _assert_songs_view_title(title):
    view_title = get_element_by_css('#song-view > h2')
    assert_text(view_title, title)
