# -*- coding: utf-8 -*-

# Authors: 
#     Leo Arias <leo.arias@canonical.com>
#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Actions for the Payments page of the Ubuntu One website."""

from datetime import datetime
from sst.actions import *
from utils import paymentperiods

def assert_page_title():
    """Assert that the title of the page is the expected."""
    assert_title(u'Ubuntu One : Checkout')

def assert_music_streaming_subscription(period):
    """Assert that the information of the new music streaming subscription 
    is the expected.                                               

    Keyword arguments:
    period -- The period of the subscription. 
              It can be 'monthly' or 'annually'.
                                                                           
    """
    if period == 'monthly':
        price = '$3.99'
    elif period == 'annually':
        price = '$39.99'
    else:
        # TODO throw an error.
        pass
    get_element(tag='h1', text='New Subscription')
    get_element(css_class='price', text=price)
    get_element(tag='span', css_class='price', text=price)
    today = datetime.utcnow()
    trial_expiration_date = paymentperiods.get_trial_expiration_date()
    get_element(tag='p', text='This is a trial subscription which will '
                              'expire %(trial_expiration)s after which your '
                              'card will automatically be billed %(price)s '  
                              '%(period)s'
                              % {'trial_expiration':
                                  trial_expiration_date.strftime('%B %-d, %Y'),
                                 'price': price, 'period': period})

def assert_monthly_music_streaming_subscription():
    """Assert that the information of the new monthly music streaming 
    subscription is the expected.

    """
    assert_music_streaming_subscription('monthly')

def assert_annually_music_streaming_subscription():
    """Assert that the information of the new yearly music streaming
    subscription is the expected.
    
    """
    assert_music_streaming_subscription('annually')

def assert_extra_storage_subscription(period, quantity):
    """Assert that the information of the new extra storage subscription 
    is the expected.
    
    Keyword arguments:
    period -- The period of the subscription.
              It can be 'monthly' or 'annually'.
    quantity -- The quantity of extra storage to buy.

    """
    if period == 'monthly':
        price_per_add_on = 2.99
        period_display = period
        period_end = paymentperiods.get_monthly_expiration_date()
    elif period == 'annually':
        price_per_add_on = 29.99
        period_display = 'yearly'
        period_end = paymentperiods.get_yearly_expiration_date()
    else:
        # TODO throw an error.                                                 
        pass
    price_per_period = quantity * price_per_add_on
    get_element(tag='h1', text='Subscription Change')
    get_element(tag='p', text='%(quantity)d x 20-Pack with 20 GB (%(period)s)'
                              % {'quantity': quantity, 
                                 'period': period_display})
    get_element(tag='strong', text=period_display.capitalize())
    get_element(css_class='price', text='$' + str(price_per_add_on))
    get_element(tag='span', css_class='price', 
                text='$' + str(price_per_period))
    today = datetime.utcnow()
    get_element(tag='p', text='This payment will cover the period from ' \
                              '%(today)s to %(period_end)s after which your ' \
                              'card will automatically be billed $%(price)s ' \
                              '%(period)s'
                              % {'today': today.strftime('%B %-d, %Y'),
                                 'period_end': 
                                     period_end.strftime('%B %-d, %Y'),
                                 'price': str(price_per_period),
                                 'period': period})

def assert_monthly_extra_storage_subscription(quantity):
    """Assert that the information of the new extra storage subscription is 
    the expected.

    Keyword arguments:
    quantity -- The quantity of extra storage to buy.

    """
    assert_extra_storage_subscription('monthly', quantity)

def assert_annually_extra_storage_subscription(quantity):
    """Assert that the information of the new extra storage subscription is 
    the expected.

    Keyword arguments:
    quantity -- The quantity of extra storage to buy.

    """
    assert_extra_storage_subscription('annually', quantity)

def set_billing_address(street='Test Street', city='Test City', 
                        state='Test State', postal_code='12345',
                        country='United States'):
    """Set the billing address information.

    Keyword arguments:
    street -- The street of the address (default 'Test Street').
    city -- The city of the address (default 'Test City').
    state -- The state or region of the address (default 'Test State').
    postal_code -- The postal code of the address (default '12345').
    country -- The country of the address (default 'United States').
    
    """
    write_textfield('id_street', street)
    write_textfield('id_city', city)
    write_textfield('id_state', state)
    write_textfield('id_postal', postal_code)
    set_dropdown_value('id_country', country)

def set_credit_card_details(card_type='Visa', name='Test Name',
                            number='4111111111111111', ccv='123',
                            expiration_month='10', expiration_year='2015'):
    """Set the credit card information.

    Default value are a valid credit card for payments on the staging 
    environment.

    Keyword arguments:
    card_type -- The type of the credit card (default 'Visa')
    name -- The name on the card (default 'Test Name')
    number -- The credit card number (default '4111111111111111')
    ccv -- The CCV number of the credit card (default '123')
    expiration_month -- The month of expiration (default '10')
    expiration_year -- The year of expiration (default '2015')

    """
    set_dropdown_value('id_card_type', card_type)
    write_textfield('id_card_holder', name)
    write_textfield('id_card_number', number)
    write_textfield('id_card_ccv', ccv)
    set_dropdown_value('id_card_expiration_0', expiration_month)
    set_dropdown_value('id_card_expiration_1', expiration_year)

def place_order_with_valid_card():
    """Place the order with a credit card that will be accepted on the
    staging environment.

    """
    set_billing_address()
    set_credit_card_details()
    agree_and_place_order()

def place_order_with_refused_card():
    """Place the order with a credit card that will be refused on the
    staging environment.
    
    """
    set_billing_address()
    set_credit_card_details(name='REFUSED')
    agree_and_place_order()


def agree_and_place_order():
    """Click the button to agree with the terms and conditions and place 
    the order.

    """
    click_button(get_element(css_class='cta', value='Purchase'))

def confirm_payment_details():
    assert_title(u'Ubuntu One : Confirm Payment Details')
    click_link(get_element(css_class='cta', href='confirmed'))
