# -*- coding: utf-8 -*-

# Copyright 2011, 2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Set up actions for Ubuntu One sst tests."""

from sst.actions import *
from config import schema
from config import environment
from data.user import User
from data.environment import Environment
import actions.header as header_actions
import actions.sso as sso_actions
import actions.services as services_actions
import actions.dashboard as dashboard_actions
import uuid
from utils import mail

# The user account for the test.
user = None

def setup(login_or_sign_up=True, new_user=False, user=None):
    config_values = schema.glue.schema_parser.values('__main__')
    set_wait_timeout(config_values.get('default_timeout'))
    _go_to_ubuntu_one_home()
    _clear_previous_session()
    if login_or_sign_up:
        _make_user_and_login_or_sign_up(new_user, user)


def _go_to_ubuntu_one_home():
    set_base_url(get_base_url())
    go_to()
    wait_for(assert_title, 'Ubuntu One : Home')


def get_base_url():
    return environment.get_environment().url


def _clear_previous_session():
    """A previous session might be started if the cache is not clean. This
    doesn't happen on Firefox because it always uses a new profile, but it
    happens on Internet Explorer because it keeps all data between executions.

    """
    if header_actions.is_logged_in():
        header_actions.logout()
        sso_actions.assert_logout()
        _go_to_ubuntu_one_home()


def _make_user_and_login_or_sign_up(new_user, user):
    if user is None:
        user = _make_user(new_user)
    _set_user(user)
    _login_or_sign_up(new_user, user)
    dashboard_actions.wait_for_page_to_load()
    header_actions.assert_login(user.full_name)


def _make_user(new_user):
    test_user = None
    if new_user:
        test_user = _make_random_user()
    else:
        test_user = _make_user_from_configuration()
    return test_user


def _make_random_user():
    random_user_uuid = str(uuid.uuid1())
    config_values = schema.glue.schema_parser.values('NewUser')
    full_name_pattern = config_values.get('full_name_pattern')
    email_address_pattern = config_values.get('email_address_pattern')
    random_full_name = full_name_pattern % random_user_uuid
    random_email_address = email_address_pattern % random_user_uuid
    password = config_values.get('password')
    return User(random_full_name, random_email_address, password)


def _make_user_from_configuration():
    config_values = schema.glue.schema_parser.values('DefaultUser')
    full_name = config_values.get('full_name')
    email_address = config_values.get('email_address')
    password = config_values.get('password')
    return User(full_name, email_address, password)


def _set_user(user_data):
    global user
    user = user_data


def get_user():
    return user


def _login_or_sign_up(new_user, user):
    if new_user:
        _sign_up(user)
    else:
        _login(user)



def _sign_up(user):
    if environment.can_create_new_users():
        header_actions.click_login_or_sign_up()
        sso_actions.create_new_account(user)
        _confirm_email(user.email)
        services_actions.agree_and_subscribe_to_free_plan()        
    else:
        skip('Can\'t create new users on this environment.')


def _confirm_email(email):
    confirmation_code = \
            mail.get_confirmation_code_for_address(email)
    sso_actions.confirm_email(confirmation_code)


def _login(user):
    header_actions.click_login_or_sign_up()
    if environment.is_development_environment():
        wait_for(assert_title, 'Ubuntu One :')
        switch_to_window()
        click_button(get_elements('button.cta[type="submit"]')[0], wait=False)
    else:
        sso_actions.login(user)
