# -*- coding: utf-8 -*-

# Copyright 2011, 2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Email reading utility copied from ISD test scripts."""

import quopri
import re
import imaplib
import time
from email import email
from config import schema

def get_confirmation_code_for_address(email_address):
    email_msg = get_latest_email_sent_to(email_address)
    vcode = None
    if email_msg:
        # The body is encoded as quoted-printable.  This affects any line
        # longer than a certain length.  Decode now to not have to worry
        # about it in the regexen.
        body = quopri.decodestring(email_msg.get_payload())
        match = re.search(
            'Here is your confirmation code:(.*)(Enter|If you made)',
            body, re.S)
        if match:
            vcode = match.group(1).strip()
        else:
            raise AssertionError("No verification link found in email.")
    return vcode


def get_latest_email_sent_to(email_address):
    with MailBox() as mbox:
        email_msg = mbox.get_latest_email_sent_to(email_address)
        mbox.delete_msgs_sent_to(email_address)
        return email_msg
                      
      
class MailBox(object):

    def __init__(self):
        config_values = schema.glue.schema_parser.values('NewUser')
        self.user = config_values.get('imap_username')
        self.password = config_values.get('imap_password')
        imap_server = config_values.get('imap_server')
        imap_port = config_values.get('imap_port')
        imap_use_ssl = config_values.get('imap_use_ssl')
        if imap_use_ssl:
            self.imap = imaplib.IMAP4_SSL(imap_server, imap_port)
        else:
            self.imap = imaplib.IMAP4(imap_server, imap_port)

    def __enter__(self):
        self.imap.login(self.user, self.password)
        return self

    def __exit__(self, type, value, traceback):
        self.imap.close()
        self.imap.logout()
        
    def get_latest_email_sent_to(self, email_address, timeout=300, poll=1):
        start_time = time.time()
        while ((time.time() - start_time) < timeout):
            # It's no use continuing until we've successfully selected
            # the inbox. And if we don't select it on each iteration
            # before searching, we get intermittent failures.
            status, data = self.imap.select('Inbox')
            if status != 'OK':
                time.sleep(poll)
                continue

            status, data = self.imap.search(None, 'TO', email_address)
            if status == 'OK':
                for num in reversed(data[0].split()):
                    status, data = self.imap.fetch(num, '(RFC822)')
                    email_msg = email.message_from_string(data[0][1])
                    return email_msg
            time.sleep(poll)

        raise AssertionError("No email sent to '%s' found in inbox "
                             "after polling for %s seconds."
                             % (email_address, timeout))
    
    def delete_msgs_sent_to(self, email_address):
        self.imap.select('Inbox')
        status, data = self.imap.search(None, 'TO', email_address)
        if status == 'OK':
            for num in reversed(data[0].split()):
                status, data = self.imap.fetch(num, '(RFC822)')
                body = data[0][1]
                self.imap.store(num, '+FLAGS', r'\Deleted')
        self.imap.expunge()
